dojo.provide("dnet.editor.Editor");

dojo.require("dijit.Editor");
dojo.require("dojo.i18n");
dojo.requireLocalization("dnet.editor", "commands", null, "ROOT,ja");

/**
 * エディターのtable拡張
 * 
 * @extends dijit.Editor
 * @class
 */
dojo.declare("dnet.editor.Editor", dijit.Editor, {
    blockNodeForEnter: "",

    /**
     * Widget作成後処理
     */
    postCreate: function() {
        var tempFunc = this.addPlugin;
        this.addPlugin = function (){return true};
        this.inherited(arguments);
        this.addPlugin = tempFunc;

        // テーブル関係機能を国際化する。
        this.mixinCommandsLocalization();

        if (this.blockNodeForEnter && dijit._editor.plugins && dnet.editor.plugins.EnterKeyHandling) {
            var _plugin = new dnet.editor.plugins.EnterKeyHandling({
                "blockNodeForEnter": this.blockNodeForEnter
            });
            this.plugins.push(_plugin);
        }

        dojo.forEach(this.plugins, this.addPlugin, this);
    },

    /**
     * コマンドのローカル追加処理
     */
    mixinCommandsLocalization: function() {

        // テーブル関係機能を国際化する。
        var commands = dojo.i18n.getLocalization("dnet.editor", "commands", this.lang);
        this.commands = dojo.mixin(this.commands, commands);
    },

    /**
     * コマンドごとのサポートUAを取得する。
     *
     * @param   {string}    command     コマンド
     * @type    {object}    サポートUA
     */
    queryCommandAvailable: function(command) {
        var plugins = ["deletetable", "editrows", "insertrow", "deleterows", "insertcol", "deletecols", "editcells", "mergecells", "splitcell"];
        var _command = command.toLowerCase();

        //[ アイコン関係を有効にする ]
        if(_command == "inserticon") {
            return {ie: true, mozilla: true, safari: false, safari420: false, opera: false};
        }

        //[ テーブル関係を有効にする ]
        if(dojo.indexOf(plugins, _command) >= 0 ) {
            return {ie: true, mozilla: true, safari: false, safari420: false, opera: false};
        }

        return this.inherited(arguments);
    },

    /**
     * コマンドが正しいかどうかを取得する。
     *
     * @param   {string}    command     コマンド
     * @type    {boolean}   正しければtrue
     */
    queryCommandEnabled: function(command) {
        var plugins = ["deletetable", "editrows", "insertrow", "deleterows", "insertcol", "deletecols", "editcells", "mergecells", "splitcell"];
        var _command = command.toLowerCase();

        //[ アイコン関係を有効にする ]
        if(_command == "inserticon") {
            return true;
        }

        //[ テーブル関係を有効にする ]
        if(dojo.indexOf(plugins, _command) >= 0 ) {
            return true;
        }

        return this.inherited(arguments);
    },

    /**
     * コマンドを実行する。
     *
     * @param   {string}    command     コマンド
     * @type    {boolean}   正しければtrue
     */
    execCommand: function(command){
        command = this._normalizeCommand(command);
        var _selection = this.document.selection;
        //[ IEで画像などに対し、inserthtmlを行った場合 ]
        if(dojo.isIE && this.customUndo && command == "inserthtml" && _selection.type.toUpperCase() == "CONTROL"){
            try {
                var insertRange = _selection.createRange();
                var cmdArgs = arguments.length > 1 ? arguments[1] : "";
                cmdArgs = this._preFilterContent(cmdArgs);
                //[ customUndo開始処理 ]
                this.endEditing();
                this._beginEditing();
                //[ 最後のControlを入れ替え ]
                while(insertRange.length > 1){
                    insertRange.remove(insertRange.item(0));
                }
                var node=insertRange.item(0);
                node.outerHTML=cmdArgs;
                this.onDisplayChanged();
                //[ customUndo終了処理 ]
                this._endEditing();
                return true;
            } catch (e) {console.log(e)}
        }

        this.inherited('execCommand',arguments);
    }
});

/**
 * 修正パッチ
 */
dijit._editor.escapeXml=function(str, noSingleQuotes){
    if (typeof(str) != "string"){
        str = new String(str);
    }

    str = str.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/"/gm, "&quot;");
    if(!noSingleQuotes){
        str = str.replace(/'/gm, "&#39;");
    }

    return str;
};

dijit.moveToBookmark=function(bookmark){
    var _document = dojo.doc;
    if(_document.selection){ // IE
        var range;
        if(dojo.isArray(bookmark)){
            range = _document.body.createControlRange();
            //[ bookmarkの各要素に関数で処理する ]
            dojo.forEach(bookmark, function(item){
                range.addElement(item);
            });
        }else{
            range = _document.selection.createRange();
            range.moveToBookmark(bookmark);
        }
        range.select();
    }else{ //Moz/W3C
        var selection = dojo.global.getSelection && dojo.global.getSelection();
        if(selection && selection.removeAllRanges){
            selection.removeAllRanges();
            selection.addRange(bookmark);
        }else{
            console.warn("No idea how to restore selection for this browser!");
        }
    }
};
